#include <Wire.h>
#include "RTClib.h"

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

#define MAIN_PIXEL_PIN        6
#define SECONDARY_PIXEL_PIN   7

#define SECONDARY_NUMPIXELS   2
#define MAIN_NUMPIXELS        12

#define M_INTENSITY           40
#define NUMZONES              13 //OFF>8h<9h>10h>11h>12h>13h>14h>15h>16h->17h>18h>OFF

#define SYNC_MODE             0
#define WAVE_MODE             1
#define MOTHER_MODE           2

//Defines the color charts
#define COLOR_OFF             {0,0,0}
#define COLOR_8               {97,196,172}
#define COLOR_9               {97,196,172}
#define COLOR_10              {112,193,101}
#define COLOR_11              {112,193,101}
#define COLOR_12              {232,233,55}
#define COLOR_13              {251,175,63}
#define COLOR_14              {242,125,173}
#define COLOR_15              {236,0,137}
#define COLOR_16              {152,106,172}
#define COLOR_17              {89,90,167}
#define COLOR_18              {99,80,130}


RTC_DS1307 realTimeClock;
Adafruit_NeoPixel secondary_pixels = Adafruit_NeoPixel(SECONDARY_NUMPIXELS, SECONDARY_PIXEL_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel main_pixels = Adafruit_NeoPixel(MAIN_NUMPIXELS, MAIN_PIXEL_PIN, NEO_GRB + NEO_KHZ800);

//Time to color chart
float colorChart[NUMZONES][3] = {COLOR_OFF,COLOR_8,COLOR_9,COLOR_10,COLOR_11,COLOR_12,COLOR_13,COLOR_14,COLOR_15,COLOR_16,COLOR_17,COLOR_18,COLOR_OFF};
//The three actual colors
float colors[3][3];
//Average brightness between M_INTENSITY and 255-M_INTENSITY
float brightness = 128;
//Small change in brightness between -M_INTENSITY and +M_INTENSITY to create a 'breathing' effect
float intensityChange[3] = {-M_INTENSITY/2 , 0 , M_INTENSITY/2};
//Gives the speed of change in the brightness for the breathing effect
float deltaIntensity[3] = {-1 , 3 , 2};
//Loudest sound intensity read
long maxSound=250000;

//Set the device up.
void setup() {
  //Serial.begin(9600);
  main_pixels.begin();
  secondary_pixels.begin();
  if (! realTimeClock.begin()) {
    while (1) {
      for (int i=0; i<12; i++) main_pixels.setPixelColor(i, main_pixels.Color(128,0,0));
      main_pixels.show();
      delay(1000);
      for (int i=0; i<12; i++) main_pixels.setPixelColor(i, main_pixels.Color(0,0,0));
      main_pixels.show();
      delay(1000);
    }
  }
  if (! realTimeClock.isrunning()) {
    realTimeClock.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }
  brightness = 255-M_INTENSITY;
}

//Compute the color corresponding to the current time
//zone is the current hour, and progress is a float between 0 and 1
//iLamp must be 0 (inner lamp), 1 (middle lamp) or 2 (outer lamps)
void gradient(int zone,float progress, int iLamp) {
  for (int iComp = 0; iComp < 3; iComp++) {
    colors[iLamp][iComp] = (1-progress)*colorChart[zone%NUMZONES][iComp] + progress*colorChart[(zone+1)%NUMZONES][iComp];
  }
}

//Compute the time zone and the progress and calls gradient for each of the three lamps
void updateColor() {
  DateTime now = realTimeClock.now();
  int zone = now.hour() - 7;
  float progress = ((float) now.minute())/60;
  
  gradient(zone-1, progress,0);
  
  if (progress-0.5 >= 0.0) gradient(zone, progress-0.5, 1);
  else gradient(zone-1, progress+0.5, 1);
  
  gradient(zone, progress,2);
}

//Update intensityChange for the lamp i
void updateIntensityChange(byte i) {
  if (intensityChange[i]+deltaIntensity[i]>M_INTENSITY||intensityChange[i]+deltaIntensity[i]<-M_INTENSITY) {
    deltaIntensity[i] = -deltaIntensity[i];
  }
  intensityChange[i] = intensityChange[i] + deltaIntensity[i];
}

//Modify the average intensity depending on the amount of noise around the device.
//It read 256 times the sound to average the noise, and 
void changeIntensity() {
  long sound = 0;
  for (int i=0; i<256; i++) {
    sound += analogRead(A0);
  }
  if (sound>maxSound) maxSound = sound;
  int relativeSound = (float(sound)/maxSound)*(255-M_INTENSITY);
  if (255*sound/maxSound > brightness && brightness < 255 - M_INTENSITY) {
    brightness += 1;
  }
  if (255*sound/maxSound < brightness && brightness > M_INTENSITY) {
    brightness -= 1;
  }
}

//Set the neopixels to the right color and update them
void updateLights() {
  for (int iPixel = 0; iPixel < 12; iPixel++) {
    main_pixels.setPixelColor(iPixel, main_pixels.Color(colors[2][0], colors[2][1], colors[2][2]));
  }
  secondary_pixels.setPixelColor(0, secondary_pixels.Color(colors[1][0], colors[1][1], colors[1][2]));
  secondary_pixels.setPixelColor(1, secondary_pixels.Color(colors[0][0], colors[0][1], colors[0][2]));
  secondary_pixels.show();
  main_pixels.show();
}

//Modulate the intensity of the three lamps to create the breathing effet and update the colors array.
//mode must be SYNC_MODE (synchronous), WAVE_MODE (wave like), MOTHER_MODE (coming from motherboard).
void modulateIntensity(byte mode) {
  switch (mode) {
  case SYNC_MODE :
    for (int iLamp = 0; iLamp < 3; iLamp++) {
      for (int iComp = 0; iComp < 3; iComp++) {
        colors[iLamp][iComp] = (brightness + intensityChange[1])/255*colors[iLamp][iComp];
      }
    }
    break;
  case WAVE_MODE :
    for (int iComp = 0; iComp < 3; iComp++) colors[0][iComp] = (brightness + intensityChange[1])/255*colors[0][iComp];
    for (int iComp = 0; iComp < 3; iComp++) colors[1][iComp] = (brightness - intensityChange[1])/255*colors[1][iComp];
    for (int iComp = 0; iComp < 3; iComp++) colors[2][iComp] = (brightness + intensityChange[1])/255*colors[2][iComp];
    break;
  case MOTHER_MODE :
    for (int iComp = 0; iComp < 3; iComp++) colors[0][iComp] = (brightness + intensityChange[1])/255*colors[0][iComp];
    for (int iComp = 0; iComp < 3; iComp++) colors[1][iComp] = (brightness + intensityChange[0])/255*colors[1][iComp];
    for (int iComp = 0; iComp < 3; iComp++) colors[2][iComp] = (brightness + intensityChange[2])/255*colors[2][iComp];
    break;
  }
}

void working() {
  //Update the colors
  updateColor();
  
  //Change the brightness to create the breathing effect
  for (byte i=0; i<3; i++) updateIntensityChange(i);
  
  //Change the main brightness depending on the sound and update the colors array
  changeIntensity();
  modulateIntensity(MOTHER_MODE);
  
  //Update the light on the device
  updateLights();
  delay(30);
}

void loop() {
  DateTime now = realTimeClock.now();
  int timer = now.hour();
  //Check of the lamp is in active hours
  if (timer >= 7 && timer <= 19) {
    working();
  }
  else delay(100000);
}
